@@ -17,18 +17,79 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-amrwbenc
+ * @see_also: #GstAmrwbDec, #GstAmrwbParse
+ *
+ * <refsect2>
+ * <para>
+ * This is an AMR wideband encoder based on the 
+ * <ulink url="http://www.penguin.cz/~utx/amr">reference codec implementation</ulink>.
+ * </para>
+ * <title>Example launch line</title>
+ * <para>
+ * <programlisting>
+ * gst-launch filesrc location=abc.wav ! wavparse ! audioresample ! audioconvert ! amrwbenc ! filesink location=abc.amr
+ * </programlisting>
+ * </para>
+ * Please not that the above stream misses the header, that is needed to play
+ * the stream.
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "gstamrwbenc.h"
 
-static const GstElementDetails gst_amrwbenc_details =
-GST_ELEMENT_DETAILS ("AMR-WB audio encoder",
-    "Codec/Encoder/Audio",
-    "Adaptive Multi-Rate Wideband audio encoder",
-    "Renato Araujo <renato.filho@indt.org.br>");
+/* these defines are not in all .h files */
+#ifndef MR660
+#define MR660  0
+#define MR885  1
+#define MR1265 2
+#define MR1425 2
+#define MR1585 3
+#define MR1825 4
+#define MR1985 5
+#define MR2305 6
+#define MR2385 7
+#define MRDTX  8
+#endif
 
+static GType
+gst_amrwbenc_bandmode_get_type ()
+{
+  static GType gst_amrwbenc_bandmode_type = 0;
+  static GEnumValue gst_amrwbenc_bandmode[] = {
+    {MR660, "MR660", "MR660"},
+    {MR885, "MR885", "MR885"},
+    {MR1265, "MR1265", "MR1265"},
+    {MR1425, "MR1425", "MR1425"},
+    {MR1585, "MR1585", "MR1585"},
+    {MR1825, "MR1825", "MR1825"},
+    {MR1985, "MR1985", "MR1985"},
+    {MR2305, "MR2305", "MR2305"},
+    {MR2385, "MR2385", "MR2385"},
+    {MRDTX, "MRDTX", "MRDTX"},
+    {0, NULL, NULL},
+  };
+  if (!gst_amrwbenc_bandmode_type) {
+    gst_amrwbenc_bandmode_type =
+        g_enum_register_static ("GstAmrwbEncBandMode", gst_amrwbenc_bandmode);
+  }
+  return gst_amrwbenc_bandmode_type;
+}
+
+#define GST_AMRWBENC_BANDMODE_TYPE (gst_amrwbenc_bandmode_get_type())
+
+#define BANDMODE_DEFAULT MR660
+
+enum
+{
+  PROP_0,
+  PROP_BANDMODE
+};
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -48,10 +109,9 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
         "rate = (int) 16000, " "channels = (int) 1")
     );
 
-static void gst_amrwbenc_base_init (gpointer klass);
-static void gst_amrwbenc_class_init (GstAmrwbEncClass * klass);
-static void gst_amrwbenc_init (GstAmrwbEnc * amrwbenc,
-    GstAmrwbEncClass * klass);
+GST_DEBUG_CATEGORY_STATIC (gst_amrwbenc_debug);
+#define GST_CAT_DEFAULT gst_amrwbenc_debug
+
 static void gst_amrwbenc_finalize (GObject * object);
 
 static GstFlowReturn gst_amrwbenc_chain (GstPad * pad, GstBuffer * buffer);
@@ -59,20 +119,63 @@ static gboolean gst_amrwbenc_setcaps (GstPad * pad, GstCaps * caps);
 static GstStateChangeReturn gst_amrwbenc_state_change (GstElement * element,
     GstStateChange transition);
 
+#define _do_init(bla) \
+    GST_DEBUG_CATEGORY_INIT (gst_amrwbenc_debug, "amrwbenc", 0, "AMR-WB audio encoder");
 
-GST_BOILERPLATE (GstAmrwbEnc, gst_amrwbenc, GstElement, GST_TYPE_ELEMENT);
+GST_BOILERPLATE_FULL (GstAmrwbEnc, gst_amrwbenc, GstElement, GST_TYPE_ELEMENT,
+    _do_init);
+
+static void
+gst_amrwbenc_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstAmrwbEnc *self = GST_AMRWBENC (object);
+
+  switch (prop_id) {
+    case PROP_BANDMODE:
+      self->bandmode = g_value_get_enum (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+
+  return;
+}
+
+static void
+gst_amrwbenc_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstAmrwbEnc *self = GST_AMRWBENC (object);
+
+  switch (prop_id) {
+    case PROP_BANDMODE:
+      g_value_set_enum (value, self->bandmode);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+
+  return;
+}
 
 static void
 gst_amrwbenc_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+  GstElementDetails details = GST_ELEMENT_DETAILS ("AMR-WB audio encoder",
+      "Codec/Encoder/Audio",
+      "Adaptive Multi-Rate Wideband audio encoder",
+      "Renato Araujo <renato.filho@indt.org.br>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template));
 
-  gst_element_class_set_details (element_class, &gst_amrwbenc_details);
+  gst_element_class_set_details (element_class, &details);
 }
 
 static void
@@ -81,28 +184,29 @@ gst_amrwbenc_class_init (GstAmrwbEncClass * klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
-
   object_class->finalize = gst_amrwbenc_finalize;
+  object_class->set_property = gst_amrwbenc_set_property;
+  object_class->get_property = gst_amrwbenc_get_property;
 
-  element_class->change_state = gst_amrwbenc_state_change;
+  g_object_class_install_property (object_class, PROP_BANDMODE,
+      g_param_spec_enum ("band-mode", "Band Mode",
+          "Encoding Band Mode (Kbps)", GST_AMRWBENC_BANDMODE_TYPE,
+          BANDMODE_DEFAULT, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+  element_class->change_state = GST_DEBUG_FUNCPTR (gst_amrwbenc_state_change);
 }
 
 static void
 gst_amrwbenc_init (GstAmrwbEnc * amrwbenc, GstAmrwbEncClass * klass)
 {
   /* create the sink pad */
-  amrwbenc->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&sink_template),
-      "sink");
+  amrwbenc->sinkpad = gst_pad_new_from_static_template (&sink_template, "sink");
   gst_pad_set_setcaps_function (amrwbenc->sinkpad, gst_amrwbenc_setcaps);
   gst_pad_set_chain_function (amrwbenc->sinkpad, gst_amrwbenc_chain);
   gst_element_add_pad (GST_ELEMENT (amrwbenc), amrwbenc->sinkpad);
 
   /* create the src pad */
-  amrwbenc->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&src_template),
-      "src");
+  amrwbenc->srcpad = gst_pad_new_from_static_template (&src_template, "src");
   gst_pad_use_fixed_caps (amrwbenc->srcpad);
   gst_element_add_pad (GST_ELEMENT (amrwbenc), amrwbenc->srcpad);
 
@@ -200,7 +304,8 @@ gst_amrwbenc_chain (GstPad * pad, GstBuffer * buffer)
     data = (guint8 *) gst_adapter_peek (amrwbenc->adapter, buffer_size);
 
     /* encode */
-    outsize = E_IF_encode (amrwbenc->handle, 0, (Word16 *) data,
+    outsize =
+        E_IF_encode (amrwbenc->handle, amrwbenc->bandmode, (Word16 *) data,
         (UWord8 *) GST_BUFFER_DATA (out), 0);
 
     gst_adapter_flush (amrwbenc->adapter, buffer_size);
