@@ -323,7 +323,8 @@ gst_faad_setcaps (GstPad * pad, GstCaps * caps)
             &channels) < 0)
       goto init_failed;
 
-    GST_DEBUG_OBJECT (faad, "channels=%u, rate=%u", channels, samplerate);
+    GST_DEBUG_OBJECT (faad, "codec_data init: channels=%u, rate=%u", channels,
+        samplerate);
 
     /* not updating these here, so they are updated in the
      * chain function, and new caps are created etc. */
@@ -339,6 +340,7 @@ gst_faad_setcaps (GstPad * pad, GstCaps * caps)
   } else if ((value = gst_structure_get_value (str, "framed")) &&
       g_value_get_boolean (value) == TRUE) {
     faad->packetised = TRUE;
+    GST_DEBUG_OBJECT (faad, "we have packetized audio");
   } else {
     faad->init = FALSE;
   }
@@ -438,13 +440,28 @@ gst_faad_chanpos_from_gst (GstAudioChannelPosition * pos, guint num)
 */
 
 static GstAudioChannelPosition *
-gst_faad_chanpos_to_gst (guchar * fpos, guint num)
+gst_faad_chanpos_to_gst (GstFaad * faad, guchar * fpos, guint num,
+    gboolean * channel_map_failed)
 {
-  GstAudioChannelPosition *pos = g_new (GstAudioChannelPosition, num);
+  GstAudioChannelPosition *pos;
   guint n;
   gboolean unknown_channel = FALSE;
 
+  *channel_map_failed = FALSE;
+
+  /* special handling for the common cases for mono and stereo */
+  if (num == 1 && fpos[0] == FRONT_CHANNEL_CENTER) {
+    GST_DEBUG_OBJECT (faad, "mono common case; won't set channel positions");
+    return NULL;
+  } else if (num == 2 && fpos[0] == FRONT_CHANNEL_LEFT
+      && fpos[1] == FRONT_CHANNEL_RIGHT) {
+    GST_DEBUG_OBJECT (faad, "stereo common case; won't set channel positions");
+    return NULL;
+  }
+
+  pos = g_new (GstAudioChannelPosition, num);
   for (n = 0; n < num; n++) {
+    GST_DEBUG_OBJECT (faad, "faad channel %d as %d", n, fpos[n]);
     switch (fpos[n]) {
       case FRONT_CHANNEL_LEFT:
         pos[n] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
@@ -478,28 +495,29 @@ gst_faad_chanpos_to_gst (guchar * fpos, guint num)
         pos[n] = GST_AUDIO_CHANNEL_POSITION_LFE;
         break;
       default:
+        GST_DEBUG_OBJECT (faad, "unknown channel %d at %d", fpos[n], n);
         unknown_channel = TRUE;
         break;
     }
   }
   if (unknown_channel) {
+    g_free (pos);
+    pos = NULL;
     switch (num) {
       case 1:{
-        GST_DEBUG ("FAAD reports unknown 1 channel mapping. Forcing to mono");
-        pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_MONO;
+        GST_DEBUG_OBJECT (faad,
+            "FAAD reports unknown 1 channel mapping. Forcing to mono");
         break;
       }
       case 2:{
-        GST_DEBUG ("FAAD reports unknown 2 channel mapping. Forcing to stereo");
-        pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
-        pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+        GST_DEBUG_OBJECT (faad,
+            "FAAD reports unknown 2 channel mapping. Forcing to stereo");
         break;
       }
       default:{
         GST_WARNING ("Unsupported FAAD channel position 0x%x encountered",
             fpos[n]);
-        g_free (pos);
-        pos = NULL;
+        *channel_map_failed = TRUE;
         break;
       }
     }
@@ -519,12 +537,13 @@ gst_faad_srcgetcaps (GstPad * pad)
   if (!supported_positions) {
     guchar *supported_fpos = g_new0 (guchar, num_supported_positions);
     gint n;
+    gboolean map_failed;
 
     for (n = 0; n < num_supported_positions; n++) {
       supported_fpos[n] = n + FRONT_CHANNEL_CENTER;
     }
-    supported_positions = gst_faad_chanpos_to_gst (supported_fpos,
-        num_supported_positions);
+    supported_positions = gst_faad_chanpos_to_gst (faad, supported_fpos,
+        num_supported_positions, &map_failed);
     g_free (supported_fpos);
   }
 
@@ -589,15 +608,18 @@ gst_faad_srcgetcaps (GstPad * pad)
         /* put channel information here */
         if (faad->channel_positions) {
           GstAudioChannelPosition *pos;
+          gboolean map_failed;
 
-          pos = gst_faad_chanpos_to_gst (faad->channel_positions,
-              faad->channels);
-          if (!pos) {
+          pos = gst_faad_chanpos_to_gst (faad, faad->channel_positions,
+              faad->channels, &map_failed);
+          if (map_failed) {
             gst_structure_free (str);
             continue;
           }
-          gst_audio_set_channel_positions (str, pos);
-          g_free (pos);
+          if (pos) {
+            gst_audio_set_channel_positions (str, pos);
+            g_free (pos);
+          }
         } else {
           gst_audio_set_structure_channel_positions_list (str,
               supported_positions, num_supported_positions);
@@ -1011,6 +1033,7 @@ gst_faad_update_caps (GstFaad * faad, faacDecFrameInfo * info)
 {
   GstAudioChannelPosition *pos;
   gboolean ret;
+  gboolean channel_map_failed;
   GstCaps *caps;
 
   /* store new negotiation information */
@@ -1029,16 +1052,21 @@ gst_faad_update_caps (GstFaad * faad, faacDecFrameInfo * info)
 
   faad->bps = 16 / 8;
 
-  pos = gst_faad_chanpos_to_gst (faad->channel_positions, faad->channels);
-  if (!pos) {
+  channel_map_failed = FALSE;
+  pos =
+      gst_faad_chanpos_to_gst (faad, faad->channel_positions, faad->channels,
+      &channel_map_failed);
+  if (channel_map_failed) {
     GST_DEBUG_OBJECT (faad, "Could not map channel positions");
     gst_caps_unref (caps);
     return FALSE;
   }
-  gst_audio_set_channel_positions (gst_caps_get_structure (caps, 0), pos);
-  g_free (pos);
+  if (pos) {
+    gst_audio_set_channel_positions (gst_caps_get_structure (caps, 0), pos);
+    g_free (pos);
+  }
 
-  GST_DEBUG ("New output caps: %" GST_PTR_FORMAT, caps);
+  GST_DEBUG_OBJECT (faad, "New output caps: %" GST_PTR_FORMAT, caps);
 
   ret = gst_pad_set_caps (faad->srcpad, caps);
   gst_caps_unref (caps);
@@ -1114,72 +1142,19 @@ gst_faad_sync (GstBuffer * buf, guint * off)
 static gboolean
 looks_like_valid_header (guint8 * input_data, guint input_size)
 {
-  guint32 rate;
-  guint32 channels;
-
-  if (input_size < 2)
+  if (input_size < 4)
     return FALSE;
 
-  rate = ((input_data[0] & 0x7) << 1) | ((input_data[1] & 0x80) >> 7);
-  channels = (input_data[1] & 0x78) >> 3;
+  if (input_data[0] == 'A'
+      && input_data[1] == 'D' && input_data[2] == 'I' && input_data[3] == 'F')
+    /* ADIF type header */
+    return TRUE;
 
-  if (rate == 0xd || rate == 0xe)       /* Reserved values */
-    return FALSE;
+  if (input_data[0] == 0xff && (input_data[1] >> 4) == 0xf)
+    /* ADTS type header */
+    return TRUE;
 
-  if (channels == 0)            /* Extended specifier: never seen one of these */
-    return FALSE;
-
-  return TRUE;
-}
-
-/*
-  clips buffer to currently configured segment. Returns FALSE if the buffer 
-  has to be dropped.
-*/
-
-static gboolean
-clip_outgoing_buffer (GstFaad * faad, GstBuffer * buffer)
-{
-  gint64 start, stop, cstart, cstop, diff;
-  gboolean res = TRUE;
-
-  if (faad->segment->format != GST_FORMAT_TIME)
-    goto beach;
-
-  start = GST_BUFFER_TIMESTAMP (buffer);
-  stop = start + GST_BUFFER_DURATION (buffer);
-
-  if (gst_segment_clip (faad->segment, GST_FORMAT_TIME,
-          start, stop, &cstart, &cstop)) {
-    diff = cstart - start;
-    if (diff > 0) {
-      GST_BUFFER_TIMESTAMP (buffer) = cstart;
-      GST_BUFFER_DURATION (buffer) -= diff;
-
-      /* time->frames->bytes */
-      diff =
-          faad->bps * faad->channels * GST_CLOCK_TIME_TO_FRAMES (diff,
-          faad->samplerate);
-      GST_BUFFER_DATA (buffer) += diff;
-      GST_BUFFER_SIZE (buffer) -= diff;
-    }
-    diff = cstop - stop;
-    if (diff > 0) {
-      GST_BUFFER_DURATION (buffer) -= diff;
-      /* time->frames->bytes */
-      diff =
-          faad->bps * faad->channels * GST_CLOCK_TIME_TO_FRAMES (diff,
-          faad->samplerate);
-      /* update size */
-      GST_BUFFER_SIZE (buffer) -= diff;
-    }
-  } else {
-    GST_DEBUG_OBJECT (faad, "buffer is outside configured segment");
-    res = FALSE;
-  }
-
-beach:
-  return res;
+  return FALSE;
 }
 
 static GstFlowReturn
@@ -1363,12 +1338,20 @@ gst_faad_chain (GstPad * pad, GstBuffer * buffer)
         }
       }
 
+      if (info.samples > G_MAXUINT / faad->bps) {
+        /* C's lovely propensity for int overflow.. */
+        GST_ELEMENT_ERROR (faad, STREAM, DECODE, (NULL),
+            ("Output buffer too large"));
+        ret = GST_FLOW_ERROR;
+        goto out;
+      }
+
       /* play decoded data */
       if (info.samples > 0) {
         guint bufsize = info.samples * faad->bps;
         guint num_samples = info.samples / faad->channels;
 
-        GST_DEBUG_OBJECT (faad, "decoded %d samples", info.samples);
+        GST_DEBUG_OBJECT (faad, "decoded %d samples", (guint) info.samples);
 
         /* note: info.samples is total samples, not per channel */
         ret =
@@ -1389,7 +1372,8 @@ gst_faad_chain (GstPad * pad, GstBuffer * buffer)
         faad->sum_dur_out += GST_BUFFER_DURATION (outbuf);
         GST_OBJECT_UNLOCK (faad);
 
-        if (clip_outgoing_buffer (faad, outbuf)) {
+        if ((outbuf = gst_audio_buffer_clip (outbuf, faad->segment,
+                    faad->samplerate, faad->bps * faad->channels))) {
           GST_LOG_OBJECT (faad,
               "pushing buffer, off=%" G_GUINT64_FORMAT ", ts=%" GST_TIME_FORMAT,
               GST_BUFFER_OFFSET (outbuf),
@@ -1397,8 +1381,7 @@ gst_faad_chain (GstPad * pad, GstBuffer * buffer)
           ret = gst_pad_push (faad->srcpad, outbuf);
           if (ret != GST_FLOW_OK)
             goto out;
-        } else
-          gst_buffer_unref (outbuf);
+        }
       }
     }
   }
@@ -1493,7 +1476,7 @@ gst_faad_change_state (GstElement * element, GstStateChange transition)
         return GST_STATE_CHANGE_FAILURE;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_segment_init (faad->segment, GST_FORMAT_UNDEFINED);
+      gst_segment_init (faad->segment, GST_FORMAT_TIME);
       break;
     default:
       break;
@@ -1533,7 +1516,7 @@ gst_faad_change_state (GstElement * element, GstStateChange transition)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  return gst_element_register (plugin, "faad", GST_RANK_SECONDARY, GST_TYPE_FAAD);
+  return gst_element_register (plugin, "faad", GST_RANK_PRIMARY, GST_TYPE_FAAD);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
