@@ -42,5 +42,4 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "gsm",
     "GSM encoder/decoder",
-    plugin_init,
-    VERSION, "LGPL", "Gstreamer", "http://gstreamer.freedesktop.org")
+    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
