@@ -21,7 +21,7 @@
  */
 
 /**
- * SECTION:element-vorbisdec
+ * SECTION:element-ivorbisdec
  * @short_description: a decoder that decodes Vorbis to raw audio
  * @see_also: vorbisenc, oggdemux
  *
@@ -72,7 +72,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
         "rate = (int) [ 1, MAX ], "
         "channels = (int) [ 1, 6 ], "
         "endianness = (int) BYTE_ORDER, "
-        "width = (int) 16, " "depth = (int) 16, " "signed = (boolean) true")
+        "width = (int) 32, " "depth = (int) 16, " "signed = (boolean) true")
     );
 
 static GstStaticPadTemplate vorbis_dec_sink_factory =
@@ -632,7 +632,7 @@ vorbis_handle_identification_packet (GstIVorbisDec * vd)
   caps = gst_caps_new_simple ("audio/x-raw-int",
       "rate", G_TYPE_INT, vd->vi.rate,
       "channels", G_TYPE_INT, vd->vi.channels,
-      "endianness", G_TYPE_INT, G_BYTE_ORDER, "width", G_TYPE_INT, 16,
+      "endianness", G_TYPE_INT, G_BYTE_ORDER, "width", G_TYPE_INT, 32,
       "depth", G_TYPE_INT, 16, "signed", G_TYPE_BOOLEAN, TRUE, NULL);
 
   if (pos) {
@@ -819,13 +819,13 @@ CLIP_TO_15 (ogg_int32_t x)
 #endif
 
 static void
-copy_samples (gint16 * out, ogg_int32_t ** in, guint samples, gint channels)
+copy_samples (gint32 * out, ogg_int32_t ** in, guint samples, gint channels)
 {
   gint i, j;
 
   for (j = 0; j < samples; j++) {
     for (i = 0; i < channels; i++) {
-      	*out++ = CLIP_TO_15 ( (in[i][j] ) >> 9);
+      *out++ = CLIP_TO_15 (in[i][j] >> 9);
     }
   }
 }
@@ -853,7 +853,7 @@ vorbis_do_clip (GstIVorbisDec * dec, GstBuffer * buf)
     /* bring clipped time to samples */
     diff = gst_util_uint64_scale_int (diff, dec->vi.rate, GST_SECOND);
     /* samples to bytes */
-    diff *= (sizeof (gint16) * dec->vi.channels);
+    diff *= (sizeof (gint32) * dec->vi.channels);
     GST_DEBUG_OBJECT (dec, "clipping start to %" GST_TIME_FORMAT " %"
         G_GUINT64_FORMAT " bytes", GST_TIME_ARGS (cstart), diff);
     GST_BUFFER_DATA (buf) += diff;
@@ -865,7 +865,7 @@ vorbis_do_clip (GstIVorbisDec * dec, GstBuffer * buf)
 
     /* bring clipped time to samples and then to bytes */
     diff = gst_util_uint64_scale_int (diff, dec->vi.rate, GST_SECOND);
-    diff *= (sizeof (gint16) * dec->vi.channels);
+    diff *= (sizeof (gint32) * dec->vi.channels);
     GST_DEBUG_OBJECT (dec, "clipping stop to %" GST_TIME_FORMAT " %"
         G_GUINT64_FORMAT " bytes", GST_TIME_ARGS (cstop), diff);
     GST_BUFFER_SIZE (buf) -= diff;
@@ -986,7 +986,7 @@ vorbis_handle_data_packet (GstIVorbisDec * vd, ogg_packet * packet)
   if ((sample_count = vorbis_synthesis_pcmout (&vd->vd, NULL)) == 0)
     goto done;
 
-  size = sample_count * vd->vi.channels * sizeof (gint16);
+  size = sample_count * vd->vi.channels * sizeof (gint32);
 
   /* alloc buffer for it */
   result =
@@ -1000,7 +1000,7 @@ vorbis_handle_data_packet (GstIVorbisDec * vd, ogg_packet * packet)
     goto wrong_samples;
 
   /* copy samples in buffer */
-  copy_samples ( (gint16 *) GST_BUFFER_DATA (out), pcm, sample_count,
+  copy_samples ((gint32 *) GST_BUFFER_DATA (out), pcm, sample_count,
       vd->vi.channels);
 
   GST_BUFFER_SIZE (out) = size;
