@@ -600,15 +600,16 @@ mpegts_packetizer_parse_nit (MpegTSPacketizer * packetizer,
 
   /* see if the buffer is large enough */
   if (descriptors_loop_length) {
+    guint8 *networkname_descriptor;
+    GstMPEGDescriptor *mpegdescriptor;
+
     if (data + descriptors_loop_length > end - 4) {
       GST_WARNING ("PID %d invalid NIT descriptors loop length %d",
           section->pid, descriptors_loop_length);
       gst_structure_free (nit);
       goto error;
     }
-    guint8 *networkname_descriptor;
-    GstMPEGDescriptor *mpegdescriptor =
-        gst_mpeg_descriptor_parse (data, descriptors_loop_length);
+    mpegdescriptor = gst_mpeg_descriptor_parse (data, descriptors_loop_length);
     networkname_descriptor =
         gst_mpeg_descriptor_find (mpegdescriptor, DESC_DVB_NETWORK_NAME);
     if (networkname_descriptor != NULL) {
@@ -676,15 +677,17 @@ mpegts_packetizer_parse_nit (MpegTSPacketizer * packetizer,
     g_free (transport_name);
 
     if (descriptors_loop_length) {
+      GstMPEGDescriptor *mpegdescriptor;
+      guint8 *delivery;
+
       if (data + descriptors_loop_length > end - 4) {
         GST_WARNING ("PID %d invalid NIT entry %d descriptors loop length %d",
             section->pid, transport_stream_id, descriptors_loop_length);
         gst_structure_free (transport);
         goto error;
       }
-      GstMPEGDescriptor *mpegdescriptor =
+      mpegdescriptor =
           gst_mpeg_descriptor_parse (data, descriptors_loop_length);
-      guint8 *delivery;
 
       if ((delivery =
               gst_mpeg_descriptor_find (mpegdescriptor,
@@ -932,6 +935,96 @@ mpegts_packetizer_parse_nit (MpegTSPacketizer * packetizer,
             "other-frequency", G_TYPE_BOOLEAN, other_frequency, NULL);
         gst_structure_set (transport, "delivery", GST_TYPE_STRUCTURE,
             delivery_structure, NULL);
+      } else if ((delivery =
+              gst_mpeg_descriptor_find (mpegdescriptor,
+                  DESC_DVB_CABLE_DELIVERY_SYSTEM))) {
+
+        guint8 *frequency_bcd =
+            DESC_DVB_CABLE_DELIVERY_SYSTEM_frequency (delivery);
+        /* see en 300 468 section 6.2.13.1 least significant bcd digit
+         * is measured in 100Hz units so multiplier needs to be 100 to get
+         * into Hz */
+        guint32 frequency = 100 *
+            ((frequency_bcd[3] & 0x0F) +
+            10 * ((frequency_bcd[3] & 0xF0) >> 4) +
+            100 * (frequency_bcd[2] & 0x0F) +
+            1000 * ((frequency_bcd[2] & 0xF0) >> 4) +
+            10000 * (frequency_bcd[1] & 0x0F) +
+            100000 * ((frequency_bcd[1] & 0xF0) >> 4) +
+            1000000 * (frequency_bcd[0] & 0x0F) +
+            10000000 * ((frequency_bcd[0] & 0xF0) >> 4));
+        guint8 modulation =
+            DESC_DVB_CABLE_DELIVERY_SYSTEM_modulation (delivery);
+        gchar *modulation_str;
+        guint8 *symbol_rate_bcd =
+            DESC_DVB_CABLE_DELIVERY_SYSTEM_symbol_rate (delivery);
+        guint32 symbol_rate =
+            (symbol_rate_bcd[2] & 0x0F) +
+            10 * ((symbol_rate_bcd[2] & 0xF0) >> 4) +
+            100 * (symbol_rate_bcd[1] & 0x0F) +
+            1000 * ((symbol_rate_bcd[1] & 0xF0) >> 4) +
+            10000 * (symbol_rate_bcd[0] & 0x0F) +
+            100000 * ((symbol_rate_bcd[0] & 0xF0) >> 4);
+        guint8 fec_inner = DESC_DVB_CABLE_DELIVERY_SYSTEM_fec_inner (delivery);
+        gchar *fec_inner_str;
+
+        switch (fec_inner) {
+          case 0:
+            fec_inner_str = "undefined";
+            break;
+          case 1:
+            fec_inner_str = "1/2";
+            break;
+          case 2:
+            fec_inner_str = "2/3";
+            break;
+          case 3:
+            fec_inner_str = "3/4";
+            break;
+          case 4:
+            fec_inner_str = "5/6";
+            break;
+          case 5:
+            fec_inner_str = "7/8";
+            break;
+          case 6:
+            fec_inner_str = "8/9";
+            break;
+          case 0xF:
+            fec_inner_str = "none";
+            break;
+          default:
+            fec_inner_str = "reserved";
+        }
+        switch (modulation) {
+          case 0x00:
+            modulation_str = "undefined";
+            break;
+          case 0x01:
+            modulation_str = "QAM16";
+            break;
+          case 0x02:
+            modulation_str = "QAM32";
+            break;
+          case 0x03:
+            modulation_str = "QAM64";
+            break;
+          case 0x04:
+            modulation_str = "QAM128";
+            break;
+          case 0x05:
+            modulation_str = "QAM256";
+            break;
+          default:
+            modulation_str = "reserved";
+        }
+        delivery_structure = gst_structure_new ("cable",
+            "modulation", G_TYPE_STRING, modulation_str,
+            "frequency", G_TYPE_UINT, frequency,
+            "symbol-rate", G_TYPE_UINT, symbol_rate,
+            "inner-fec", G_TYPE_STRING, fec_inner_str, NULL);
+        gst_structure_set (transport, "delivery", GST_TYPE_STRUCTURE,
+            delivery_structure, NULL);
       }
       if ((delivery =
               gst_mpeg_descriptor_find (mpegdescriptor,
@@ -944,10 +1037,10 @@ mpegts_packetizer_parse_nit (MpegTSPacketizer * packetizer,
           GstStructure *channel;
           GValue channel_value = { 0 };
           guint16 service_id = GST_READ_UINT16_BE (current_pos);
+          guint16 logical_channel_number;
 
           current_pos += 2;
-          guint16 logical_channel_number =
-              GST_READ_UINT16_BE (current_pos) & 0x03ff;
+          logical_channel_number = GST_READ_UINT16_BE (current_pos) & 0x03ff;
           channel =
               gst_structure_new ("channels", "service-id", G_TYPE_UINT,
               service_id, "logical-channel-number", G_TYPE_UINT,
@@ -1109,14 +1202,16 @@ mpegts_packetizer_parse_sdt (MpegTSPacketizer * packetizer,
     g_free (service_name);
 
     if (descriptors_loop_length) {
+      guint8 *service_descriptor;
+      GstMPEGDescriptor *mpegdescriptor;
+
       if (data + descriptors_loop_length > end - 4) {
         GST_WARNING ("PID %d invalid SDT entry %d descriptors loop length %d",
             section->pid, service_id, descriptors_loop_length);
         gst_structure_free (service);
         goto error;
       }
-      guint8 *service_descriptor;
-      GstMPEGDescriptor *mpegdescriptor =
+      mpegdescriptor =
           gst_mpeg_descriptor_parse (data, descriptors_loop_length);
       service_descriptor =
           gst_mpeg_descriptor_find (mpegdescriptor, DESC_DVB_SERVICE);
@@ -1333,15 +1428,17 @@ mpegts_packetizer_parse_eit (MpegTSPacketizer * packetizer,
     g_free (event_name);
 
     if (descriptors_loop_length) {
+      guint8 *event_descriptor;
+      GArray *component_descriptors;
+      GstMPEGDescriptor *mpegdescriptor;
+
       if (data + descriptors_loop_length > end - 4) {
         GST_WARNING ("PID %d invalid EIT descriptors loop length %d",
             section->pid, descriptors_loop_length);
         gst_structure_free (event);
         goto error;
       }
-      guint8 *event_descriptor;
-      GArray *component_descriptors;
-      GstMPEGDescriptor *mpegdescriptor =
+      mpegdescriptor =
           gst_mpeg_descriptor_parse (data, descriptors_loop_length);
       event_descriptor =
           gst_mpeg_descriptor_find (mpegdescriptor, DESC_DVB_SHORT_EVENT);
