@@ -21,7 +21,7 @@
 **
 ** VRCVII sound hardware emulation
 ** Thanks to Charles MacDonald (cgfm2@hooked.net) for donating code.
-** $Id: vrc7_snd.c,v 1.2 2006-10-17 11:04:14 tpm Exp $
+** $Id: vrc7_snd.c,v 1.4 2008-03-26 07:40:56 slomo Exp $
 */
 
 #include <stdio.h>
@@ -138,11 +138,9 @@ vrc7_init (void)
 static void
 vrc7_shutdown (void)
 {
-  void *t = buffer;
-
   vrc7_reset ();
   OPLDestroy (vrc7.ym3812);
-  free (t);
+  free (buffer);
 }
 
 /* channel (0-9), instrument (0-F), volume (0-3F, YM3812 format) */
@@ -326,19 +324,21 @@ apuext_t vrc7_ext = {
 
 /*
 ** $Log: vrc7_snd.c,v $
-** Revision 1.2  2006-10-17 11:04:14  tpm
-** Patch by: Josep Torra Valles  <josep at fluendo com>
+** Revision 1.4  2008-03-26 07:40:56  slomo
+** * gst/nsf/Makefile.am:
 ** * gst/nsf/fds_snd.c:
 ** * gst/nsf/mmc5_snd.c:
 ** * gst/nsf/nsf.c:
+** * gst/nsf/types.h:
 ** * gst/nsf/vrc7_snd.c:
 ** * gst/nsf/vrcvisnd.c:
-** Fix some things the Forte compiler warns about (#362626).
+** * gst/nsf/memguard.c:
+** * gst/nsf/memguard.h:
+** Remove memguard again and apply hopefully all previously dropped
+** local patches. Should be really better than the old version now.
 **
-** Revision 1.1  2006/07/13 15:07:28  wtay
-** Based on patches by: Johan Dahlin <johan at gnome dot org>
-** Ronald Bultje <rbultje at ronald dot bitfreak dot net>
-** * configure.ac:
+** Revision 1.3  2008-03-25 15:56:12  slomo
+** Patch by: Andreas Henriksson <andreas at fatal dot set>
 ** * gst/nsf/Makefile.am:
 ** * gst/nsf/dis6502.h:
 ** * gst/nsf/fds_snd.c:
@@ -346,7 +346,6 @@ apuext_t vrc7_ext = {
 ** * gst/nsf/fmopl.c:
 ** * gst/nsf/fmopl.h:
 ** * gst/nsf/gstnsf.c:
-** * gst/nsf/gstnsf.h:
 ** * gst/nsf/log.c:
 ** * gst/nsf/log.h:
 ** * gst/nsf/memguard.c:
@@ -365,7 +364,13 @@ apuext_t vrc7_ext = {
 ** * gst/nsf/vrc7_snd.h:
 ** * gst/nsf/vrcvisnd.c:
 ** * gst/nsf/vrcvisnd.h:
-** Added NSF decoder plugin. Fixes 151192.
+** Update our internal nosefart to nosefart-2.7-mls to fix segfaults
+** on some files. Fixes bug #498237.
+** Remove some // comments, fix some compiler warnings and use pow()
+** instead of a slow, selfmade implementation.
+**
+** Revision 1.1  2003/04/08 20:53:01  ben
+** Adding more files...
 **
 ** Revision 1.5  2000/07/04 04:51:02  matt
 ** made data types stricter
