@@ -18,6 +18,31 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-speed
+ *
+ * <refsect2>
+ * <para>
+ * Plays an audio stream at a different speed.
+ * </para>
+ * <para>
+ * Do not use this element. Either use the 'pitch' element, or do a seek with
+ * a non-1.0 rate parameter, this will have the same effect as using the speed
+ * element (but relies on the decoder/demuxer to handle this correctly, also
+ * requires a fairly up-to-date gst-plugins-base, as of February 2007).
+ * </para>
+ * <title>Example launch line</title>
+ * <para>
+ * <programlisting>
+ * gst-launch filesrc location=test.ogg ! decodebin ! audioconvert ! speed speed=1.5 ! audioconvert ! audioresample ! autoaudiosink
+ * </programlisting>
+ * Plays an .ogg file at 1.5x speed.
+ * </para>
+ * </refsect2>
+ *
+ * Last reviewed on 2007-02-26 (0.10.4.1)
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -29,6 +54,9 @@
 
 #include "gstspeed.h"
 
+GST_DEBUG_CATEGORY_STATIC (speed_debug);
+#define GST_CAT_DEFAULT speed_debug
+
 /* elementfactory information */
 static const GstElementDetails speed_details = GST_ELEMENT_DETAILS ("Speed",
     "Filter/Effect/Audio",
@@ -193,47 +221,41 @@ static gboolean
 speed_src_event (GstPad * pad, GstEvent * event)
 {
   GstSpeed *filter;
-  gboolean ret = TRUE;
+  gboolean ret = FALSE;
 
   filter = GST_SPEED (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_SEEK:
-    {
-
+    case GST_EVENT_SEEK:{
       gdouble rate;
       GstFormat format;
       GstSeekFlags flags;
       GstSeekType start_type, stop_type;
       gint64 start, stop;
 
-      gst_event_parse_seek (event, &rate, &format, &flags,
-          &start_type, &start, &stop_type, &stop);
-      switch (format) {
-        case GST_FORMAT_DEFAULT:
-          /* fall through */
-        case GST_FORMAT_BYTES:
-          /* fall through */
-        case GST_FORMAT_TIME:
-          gst_event_unref (event);
+      gst_event_parse_seek (event, &rate, &format, &flags, &start_type, &start,
+          &stop_type, &stop);
+      gst_event_unref (event);
 
-          if (start_type != GST_SEEK_TYPE_NONE) {
-            start *= filter->speed;
-          }
-
-          if (stop_type != GST_SEEK_TYPE_NONE) {
-            stop *= filter->speed;
-          }
-
-          event = gst_event_new_seek (rate, format, flags, start_type, start,
-              stop_type, stop);
-
-          ret = gst_pad_send_event (GST_PAD_PEER (filter->sinkpad), event);
-          break;
-        default:
-          break;
+      if (format != GST_FORMAT_TIME) {
+        GST_DEBUG_OBJECT (filter, "only support seeks in TIME format");
+        break;
       }
 
+      if (start_type != GST_SEEK_TYPE_NONE && start != -1) {
+        start *= filter->speed;
+      }
+
+      if (stop_type != GST_SEEK_TYPE_NONE && stop != -1) {
+        stop *= filter->speed;
+      }
+
+      event = gst_event_new_seek (rate, format, flags, start_type, start,
+          stop_type, stop);
+
+      GST_LOG ("sending seek event: %" GST_PTR_FORMAT, event->structure);
+
+      ret = gst_pad_send_event (GST_PAD_PEER (filter->sinkpad), event);
       break;
     }
     default:
@@ -336,7 +358,6 @@ speed_src_query (GstPad * pad, GstQuery * query)
       GstFormat format;
       GstFormat rformat = GST_FORMAT_TIME;
       gint64 cur;
-      GstPad *peer;
       GstFormat conv_format = GST_FORMAT_TIME;
 
       /* save requested format */
@@ -345,15 +366,10 @@ speed_src_query (GstPad * pad, GstQuery * query)
       /* query peer for current position in time */
       gst_query_set_position (query, GST_FORMAT_TIME, -1);
 
-      if ((peer = gst_pad_get_peer (filter->sinkpad)) == NULL)
-        goto error;
-
-      if (!gst_pad_query_position (peer, &rformat, &cur)) {
+      if (!gst_pad_query_peer_position (filter->sinkpad, &rformat, &cur)) {
         GST_LOG_OBJECT (filter, "query on peer pad failed");
-        gst_object_unref (peer);
         goto error;
       }
-      gst_object_unref (peer);
 
       if (rformat == GST_FORMAT_BYTES)
         GST_LOG_OBJECT (filter, "peer pad returned current=%lld bytes", cur);
@@ -386,7 +402,6 @@ speed_src_query (GstPad * pad, GstQuery * query)
       GstFormat format;
       GstFormat rformat = GST_FORMAT_TIME;
       gint64 end;
-      GstPad *peer;
       GstFormat conv_format = GST_FORMAT_TIME;
 
       /* save requested format */
@@ -395,15 +410,10 @@ speed_src_query (GstPad * pad, GstQuery * query)
       /* query peer for total length in time */
       gst_query_set_duration (query, GST_FORMAT_TIME, -1);
 
-      if ((peer = gst_pad_get_peer (filter->sinkpad)) == NULL)
-        goto error;
-
-      if (!gst_pad_query_duration (peer, &rformat, &end)) {
+      if (!gst_pad_query_peer_duration (filter->sinkpad, &rformat, &end)) {
         GST_LOG_OBJECT (filter, "query on peer pad failed");
-        gst_object_unref (peer);
         goto error;
       }
-      gst_object_unref (peer);
 
       if (rformat == GST_FORMAT_BYTES)
         GST_LOG_OBJECT (filter, "peer pad returned total=%lld bytes", end);
@@ -480,16 +490,14 @@ static void
 speed_init (GstSpeed * filter)
 {
   filter->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_speed_sink_template), "sink");
+      gst_pad_new_from_static_template (&gst_speed_sink_template, "sink");
   gst_pad_set_setcaps_function (filter->sinkpad, speed_setcaps);
   gst_pad_set_chain_function (filter->sinkpad, speed_chain);
   gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
   gst_pad_set_event_function (filter->sinkpad, speed_sink_event);
 
   filter->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_speed_src_template), "src");
+      gst_pad_new_from_static_template (&gst_speed_src_template, "src");
   gst_pad_set_setcaps_function (filter->srcpad, speed_setcaps);
   gst_pad_set_query_type_function (filter->srcpad, speed_get_query_types);
   gst_pad_set_query_function (filter->srcpad, speed_src_query);
@@ -573,87 +581,47 @@ speed_chain_float32 (GstSpeed * filter, GstBuffer * in_buf, GstBuffer * out_buf,
 static gboolean
 speed_sink_event (GstPad * pad, GstEvent * event)
 {
-
-
   GstSpeed *filter;
-  gboolean ret;
+  gboolean ret = FALSE;
 
   filter = GST_SPEED (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_NEWSEGMENT:
-    {
-
+    case GST_EVENT_NEWSEGMENT:{
       gdouble rate;
       gboolean update = FALSE;
-      GstFormat format, conv_format;
+      GstFormat format;
       gint64 start_value, stop_value, base;
 
       gst_event_parse_new_segment (event, &update, &rate, &format, &start_value,
           &stop_value, &base);
+      gst_event_unref (event);
+
+      if (format != GST_FORMAT_TIME) {
+        GST_WARNING_OBJECT (filter, "newsegment event not in TIME format!");
+        break;
+      }
 
       g_assert (filter->speed > 0);
 
-      start_value /= filter->speed;
-      stop_value /= filter->speed;
+      if (start_value >= 0)
+        start_value /= filter->speed;
+      if (stop_value >= 0)
+        stop_value /= filter->speed;
+      base /= filter->speed;
 
-      if (format == GST_FORMAT_TIME) {
+      /* this would only really be correct if we clipped incoming data */
+      filter->timestamp = start_value;
 
-        conv_format = GST_FORMAT_BYTES;
-
-        filter->timestamp = start_value;
-        ret = gst_speed_convert (pad, GST_FORMAT_TIME, filter->timestamp,
-            &conv_format, &filter->offset);
-
-
-      } else if (format == GST_FORMAT_BYTES) {
-
-        conv_format = GST_FORMAT_TIME;
-
-        filter->offset = start_value;
-        ret = gst_speed_convert (pad, GST_FORMAT_BYTES, filter->offset,
-            &conv_format, &filter->timestamp);
-
-
-      } else if (format == GST_FORMAT_DEFAULT) {
-
-        conv_format = GST_FORMAT_TIME;
-
-        ret = gst_speed_convert (pad, GST_FORMAT_BYTES, start_value,
-            &conv_format, &filter->timestamp);
-
-        conv_format = GST_FORMAT_BYTES;
-
-        ret = gst_speed_convert (pad, GST_FORMAT_TIME, start_value,
-            &conv_format, &filter->offset);
-
-      }
-
-      gst_event_unref (event);
-      event =
-          gst_event_new_new_segment (update, rate, format, start_value,
-          stop_value, base);
-
-
-      if (!(ret = gst_pad_event_default (pad, event))) {
-        gst_event_unref (event);
-      }
-
-      ret = TRUE;
+      /* set to NONE so it gets reset later based on the timestamp when we have
+       * the samplerate */
+      filter->offset = GST_BUFFER_OFFSET_NONE;
 
+      ret =
+          gst_pad_event_default (pad, gst_event_new_new_segment (update, rate,
+              format, start_value, stop_value, base));
       break;
     }
-
-    case GST_EVENT_EOS:
-
-      if (!(ret = gst_pad_event_default (pad, event))) {
-        gst_event_unref (event);
-      }
-
-      ret = TRUE;
-
-      break;
-
     default:
       ret = gst_pad_event_default (pad, event);
       break;
@@ -673,27 +641,31 @@ speed_chain (GstPad * pad, GstBuffer * in_buf)
   GstBuffer *out_buf;
   GstSpeed *filter;
   guint c, in_samples, out_samples, out_size;
-  GstFlowReturn result = GST_FLOW_OK;
+  GstFlowReturn flow;
 
-  g_return_val_if_fail (pad != NULL, GST_FLOW_ERROR);
-  g_return_val_if_fail (GST_IS_PAD (pad), GST_FLOW_ERROR);
+  filter = GST_SPEED (GST_PAD_PARENT (pad));
 
-  filter = GST_SPEED (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (filter->sample_size == 0 || filter->rate == 0)) {
+    flow = GST_FLOW_NOT_NEGOTIATED;
+    goto done;
+  }
+
+  if (G_UNLIKELY (filter->offset == GST_BUFFER_OFFSET_NONE)) {
+    filter->offset =
+        gst_util_uint64_scale_int (filter->timestamp, filter->rate, GST_SECOND);
+  }
 
   /* buffersize has to be aligned by samplesize */
   out_size = ceil ((gfloat) GST_BUFFER_SIZE (in_buf) / filter->speed);
   out_size = ((out_size + filter->sample_size - 1) / filter->sample_size) *
       filter->sample_size;
 
-  result =
+  flow =
       gst_pad_alloc_buffer_and_set_caps (filter->srcpad, -1, out_size,
       GST_PAD_CAPS (filter->srcpad), &out_buf);
 
-  if (result != GST_FLOW_OK) {
-    gst_buffer_unref (in_buf);
-    gst_object_unref (filter);
-    return result;
-  }
+  if (flow != GST_FLOW_OK)
+    goto done;
 
   in_samples = GST_BUFFER_SIZE (in_buf) / filter->sample_size;
 
@@ -714,17 +686,22 @@ speed_chain (GstPad * pad, GstBuffer * in_buf)
   GST_BUFFER_TIMESTAMP (out_buf) = filter->timestamp;
 
   filter->offset += GST_BUFFER_SIZE (out_buf) / filter->sample_size;
-  filter->timestamp = filter->offset * GST_SECOND / filter->rate;
+  filter->timestamp =
+      gst_util_uint64_scale_int (filter->offset, GST_SECOND, filter->rate);
 
+  /* make sure it's at least nominally a perfect stream */
   GST_BUFFER_DURATION (out_buf) =
       filter->timestamp - GST_BUFFER_TIMESTAMP (out_buf);
 
-  result = gst_pad_push (filter->srcpad, out_buf);
+  flow = gst_pad_push (filter->srcpad, out_buf);
+
+done:
+
+  if (G_UNLIKELY (flow != GST_FLOW_OK))
+    GST_DEBUG_OBJECT (filter, "flow: %s", gst_flow_get_name (flow));
 
   gst_buffer_unref (in_buf);
-  gst_object_unref (filter);
-  return result;
-
+  return flow;
 }
 
 static void
@@ -768,7 +745,7 @@ speed_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      speed->offset = 0;
+      speed->offset = GST_BUFFER_OFFSET_NONE;
       speed->timestamp = 0;
       break;
     default:
@@ -784,6 +761,8 @@ speed_change_state (GstElement * element, GstStateChange transition)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  GST_DEBUG_CATEGORY_INIT (speed_debug, "speed", 0, "speed element");
+
   return gst_element_register (plugin, "speed", GST_RANK_NONE, GST_TYPE_SPEED);
 }
 
